      PROGRAM PBLTEST
*
*     PBLAS example code
*
      INCLUDE            'matsize.f'
      INTEGER            DLEN_, LLD_
      PARAMETER          ( DLEN_ = 9, LLD_ = 9 )
      DOUBLE PRECISION   ONE
      PARAMETER          ( ONE = 1.0D+0 )
*
      CHARACTER*80       MATFILE, RHSFILE, SOLFILE
      INTEGER            IAM, ICTXT, INFO, IPA, IPB,
     $                   IPW, MYCOL, MYROW, NPCOL,
     $                   NPROCS, NPROW, NP, NQ, NQRHS
*
      INTEGER            DESCA( DLEN_ ), DESCB( DLEN_ ), DESCC( DLEN_ )
      DOUBLE PRECISION   MEM( MEMSIZ )
      DOUBLE PRECISION   BNORM, CNORM, PDLANGE
*
      EXTERNAL           BLACS_EXIT, BLACS_GET, BLACS_GRIDEXIT,
     $                   BLACS_GRIDINFO, BLACS_GRIDINIT, BLACS_PINFO,
     $                   DESCINIT, PDGEMM, PDLAPRNT, PDLAREAD, PDLAWRITE
*
      INTEGER            NUMROC
      EXTERNAL           NUMROC
*
      INTRINSIC          MAX
*
*
      CALL BLACS_PINFO( IAM, NPROCS )
      IF( IAM.EQ.0 ) THEN
         WRITE( 6, * ) '***********************************************'
         WRITE( 6, * ) 'Example of PBLAS routine call: (PDGEMM)'
         WRITE( 6, * ) 'C = A * B - C' 
         WRITE( 6, * ) '***********************************************'
      END IF
*
      MATFILE = 'SCAEXMAT.dat'
      RHSFILE = 'SCAEXRHS.dat'
      SOLFILE = 'SCAEXSOL.dat'
*
*     Define process grid
*
      NPROW = INT(SQRT(REAL(NPROCS)))                
      NPCOL = NPROCS / NPROW 
      IF( IAM.EQ.0 ) THEN
         WRITE( 6, * )
         WRITE( 6, * ) 
         WRITE( 6, 9999 ) NPCOL, NPROW 
 9999    FORMAT('Process Grid:',I3'  x',I3) 
         WRITE( 6, * )
      END IF
      CALL BLACS_GET( -1, 0, ICTXT )
      CALL BLACS_GRIDINIT( ICTXT, 'Row-major', NPROW, NPCOL )
      CALL BLACS_GRIDINFO( ICTXT, NPROW, NPCOL, MYROW, MYCOL )
*
*     Go to bottom of process grid loop if this case doesn't use my
*     process
*
      IF( MYROW.GE.NPROW .OR. MYCOL.GE.NPCOL ) GO TO 20
*
      NP    = NUMROC( N, NB, MYROW, 0, NPROW )
      NQ    = NUMROC( N, NB, MYCOL, 0, NPCOL )
      NQRHS = NUMROC( NRHS, NB, MYCOL, 0, NPCOL )
*
*     Initialize the array descriptor for the matrix A and B
*
      CALL DESCINIT(DESCA, N, N, NB, NB, 0, 0, ICTXT, MAX(1,NP), INFO)
      CALL DESCINIT(DESCB, N, NRHS, NB, NB, 0, 0, ICTXT,MAX(1,NP), INFO)
      CALL DESCINIT(DESCC, N, NRHS, NB, NB, 0, 0, ICTXT,MAX(1,NP), INFO)
*
*     Assign pointers into MEM for PBLAS arrays, A is
*     allocated starting at position MEM( 1 )
*
      IPA = 1
      IPB = IPA + DESCA( LLD_ )*NQ
      IPC = IPB + DESCB( LLD_ )*NQRHS
      IPW = IPC + DESCB( LLD_ )*NQRHS
*
*     Read from file and distribute matrices A, B and C
*
      CALL PDLAREAD( MATFILE, MEM( IPA ), DESCA, 0, 0, MEM( IPW ) )
      CALL PDLAREAD( SOLFILE, MEM( IPB ), DESCB, 0, 0, MEM( IPW ) )
      CALL PDLAREAD( RHSFILE, MEM( IPC ), DESCC, 0, 0, MEM( IPW ) )
*
*     Call PBLAS PDGEMM routine
*
      IF( IAM.EQ.0 ) THEN
         WRITE( 6, * )
         WRITE( 6, * ) 'Matrix A:'
         WRITE( 6, * )
      END IF
      CALL PDLAPRNT( N, N, MEM(IPA), 1, 1, DESCA, 0, 0, 'A', 6,MEM(IPW))
      IF( IAM.EQ.0 ) THEN
         WRITE( 6, * )
         WRITE( 6, * ) 'Matrix B:'
         WRITE( 6, * )
      END IF
      CALL PDLAPRNT( N,NRHS,MEM(IPB),1,1,DESCB,0,0,'B',6,MEM(IPW) )
*
      CALL PDGEMM( 'No transpose', 'No transpose', N, NRHS, N, ONE,
     $             MEM( IPA ), 1, 1, DESCA, MEM( IPB ), 1, 1, DESCB,
     $             -ONE, MEM( IPC ), 1, 1, DESCC )
*
      IF( MYROW.EQ.0 .AND. MYCOL.EQ.0 ) THEN
         WRITE( 6, * )
         WRITE( 6, * ) 'Matrix C = A * B - C:'
         WRITE( 6, * )
      END IF
      CALL PDLAPRNT( N,NRHS,MEM(IPC),1,1,DESCC,0,0,'C',6,MEM(IPW) )
*
      BNORM = PDLANGE( 'I', N, NRHS, MEM(IPB), 1, 1, DESCB, MEM(IPW) )
      CNORM = PDLANGE( 'I', N, NRHS, MEM(IPC), 1, 1, DESCC, MEM(IPW) )
      IF( MYROW.EQ.0 .AND. MYCOL.EQ.0 ) THEN
         WRITE( 6, * )
         WRITE( 6, 9998 ) 'B',BNORM,'C',CNORM
 9998    FORMAT('Norm Matrix ',A,':',D15.8) 
         WRITE( 6, * )
      END IF
*
      CALL BLACS_GRIDEXIT( ICTXT )
*
   20 CONTINUE
*
      CALL BLACS_EXIT( 0 )
*
      STOP
      END
