      PROGRAM SLPTEST
*
*     ScaLAPACK example code
*
*     .. Parameters ..
      INCLUDE            'matsize.f'
      INTEGER            DLEN_, LLD_
      PARAMETER          ( DLEN_ = 9, LLD_ = 9 )
*     ..
*     .. Local Scalars ..
      CHARACTER*80       MATFILE, RHSFILE, SOLFILE
      INTEGER            IAM, ICTXT, INFO, IPA, IPB, IPPIV,
     $                   IPW, MYCOL, MYROW, NPCOL,
     $                   NPROCS, NPROW, NP, NQ, NQRHS
     $                   NP1, NQ1, NQRHS1
*     ..
*     .. Local Arrays ..
      INTEGER            DESCA( DLEN_ ), DESCB( DLEN_ )
      DOUBLE PRECISION   MEM( MEMSIZ )
      INTEGER            ISEED(4)
*     ..
*     .. External Subroutines ..
      EXTERNAL           BLACS_EXIT, BLACS_GET, BLACS_GRIDEXIT,
     $                   BLACS_GRIDINFO, BLACS_GRIDINIT, BLACS_PINFO,
     $                   DESCINIT, PDGESV, PDLAPRNT, PDLAREAD, PDLAWRITE
*     ..
*     .. External Functions ..
      INTEGER            NUMROC
      EXTERNAL           NUMROC
*     ..
*     .. Intrinsic Functions ..
      INTRINSIC          MAX
*     ..
*     .. Executable Statements ..
*
      CALL BLACS_PINFO( IAM, NPROCS )
      IF( IAM.EQ.0 ) THEN
         WRITE( 6, * ) '***********************************************'
         WRITE( 6, * ) 'Example of ScaLAPACK routine call: (PDGESV)'
         WRITE( 6, * ) 'A * X = B'
         WRITE( 6, * ) '***********************************************'
      END IF
*
      MATFILE = 'SCAEXMAT.dat'
      RHSFILE = 'SCAEXRHS.dat'
      SOLFILE = 'SCAEXSOL.dat'
*
*     Define process grid
*
      NPROW = INT(SQRT(REAL(NPROCS)))                
      NPCOL = NPROCS / NPROW 
      IF( IAM.EQ.0 ) THEN
         WRITE( 6, * )
         WRITE( 6, * ) 
         WRITE( 6, 9999 ) NPCOL, NPROW 
 9999    FORMAT('Process Grid:',I3'  x',I3) 
         WRITE( 6, * )
      END IF
      CALL BLACS_GET( -1, 0, ICTXT )
      CALL BLACS_GRIDINIT( ICTXT, 'Row-major', NPROW, NPCOL )
      CALL BLACS_GRIDINFO( ICTXT, NPROW, NPCOL, MYROW, MYCOL )
*
*     Go to bottom of process grid loop if this case doesn't use my
*     process
*
      IF( MYROW.GE.NPROW .OR. MYCOL.GE.NPCOL ) GO TO 20
*
      NP    = NUMROC( N, NB, MYROW, 0, NPROW )
      NQ    = NUMROC( N, NB, MYCOL, 0, NPCOL )
      NQRHS = NUMROC( NRHS, NB, MYCOL, 0, NPCOL )
*
*     Initialize the array descriptor for the matrix A and B
*
      CALL DESCINIT(DESCA, N, N, NB, NB, 0, 0, ICTXT, MAX(1,NP), INFO)
      CALL DESCINIT(DESCB, N, NRHS, NB, NB, 0, 0, ICTXT,MAX(1,NP), INFO)
*
*     Assign pointers into MEM for SCALAPACK arrays, A is
*     allocated starting at position MEM( 1 )
*
      IPA = 1
      IPB = IPA + DESCA( LLD_ )*NQ
      IPPIV = IPB + DESCB( LLD_ )*NQRHS
      IPDA = IPPIV + NP
      IPDB = IPDA + MIN(NP,NQ)
      IPW = IPDB + MIN(NP,NQRHS)
*
*     Read from file and distribute matrices A and B
*
*
      CALL PDLAREAD( MATFILE, MEM( IPA ), DESCA, 0, 0, MEM( IPW ) )
      CALL PDLAREAD( RHSFILE, MEM( IPB ), DESCB, 0, 0, MEM( IPW ) )
*
*     Call ScaLAPACK PDGESV routine
*
      IF( IAM.EQ.0 ) THEN
         WRITE( 6, * )
         WRITE( 6, * ) 'A * X = B, Matrix A:'
         WRITE( 6, * )
      END IF
      CALL PDLAPRNT( N, N, MEM(IPA), 1, 1, DESCA, 0, 0, 'A', 6,MEM(IPW))
      IF( IAM.EQ.0 ) THEN
         WRITE( 6, * )
         WRITE( 6, * ) 'Matrix B:'
         WRITE( 6, * )
      END IF
      CALL PDLAPRNT( N,NRHS,MEM(IPB),1,1,DESCB,0,0,'B',6,MEM(IPW) )
*
      CALL PDGESV( N, NRHS, MEM( IPA ), 1, 1, DESCA, MEM( IPPIV ),
     $             MEM( IPB ), 1, 1, DESCB, INFO )
*
      IF( MYROW.EQ.0 .AND. MYCOL.EQ.0 ) THEN
         WRITE( 6, * )
         WRITE( 6, * ) 'Matrix X = A^{-1} * B:'
         WRITE( 6, * )
      END IF
      CALL PDLAPRNT( N,NRHS,MEM(IPB),1,1,DESCB,0,0,'X',6,MEM(IPW) )
      CALL PDLAWRITE( SOLFILE,N,NRHS,MEM(IPB),1,1,DESCB,0,0,MEM(IPW) )
*
      CALL BLACS_GRIDEXIT( ICTXT )
*
   20 CONTINUE
*
      CALL BLACS_EXIT( 0 )
*
      STOP
      END
