#include <cmath>
#include <iostream>
#include <mpi.h>

using namespace std;

double f(double x)
{
  return (4.0/(1.0+x*x));
}
int main(int argc, char* argv[])
{
  int rank, size;
  MPI_Init( &argc, &argv);
  MPI_Comm_rank( MPI_COMM_WORLD, &rank);
  MPI_Comm_size( MPI_COMM_WORLD, &size);
  int i, N=100;
  double delta_x=1.0/(double (N));
  double pi_estimate, pi_partial=0.0;
  double x_;
  for(i=rank; i<N; i+=size)
  {
    x_ = ((double (i))+0.5)*delta_x;
    pi_partial += f(x_);
  }
  pi_partial *= delta_x;
  MPI_Reduce(&pi_partial, &pi_estimate, 1, MPI_DOUBLE, MPI_SUM, 0, MPI_COMM_WORLD );
  if( rank == 0 )
  {
  const double pi_exact=3.141592653589793238462643;
  double pi_diff=fabs(pi_exact - pi_estimate);
  printf("pi_exact    = %24.16f\n", pi_exact);
  printf("pi_estimate = %24.16f\n", pi_estimate);
  printf("pi_diff     = %24.16f\n", pi_diff);
  }
  MPI_Finalize();
  return 0;
}
